@@ -24,6 +24,7 @@
 #include <EGL/eglext.h>
 #include <fcntl.h>
 #include <libdrm/drm_fourcc.h>
+#include <unistd.h>
 
 #if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
 #include <QtPlatformHeaders/QEGLNativeContext>
@@ -39,7 +40,7 @@ class PipeWireSourceItemPrivate
 {
 public:
     uint m_nodeId = 0;
-    uint m_fd = 0;
+    std::optional<uint> m_fd = 0;
     std::function<QSGTexture *()> m_createNextTexture;
     QScopedPointer<PipeWireSourceStream> m_stream;
     QScopedPointer<QOpenGLTexture> m_texture;
@@ -94,6 +95,9 @@ PipeWireSourceItem::PipeWireSourceItem(QQuickItem *parent)
 
 PipeWireSourceItem::~PipeWireSourceItem()
 {
+    if (d->m_fd) {
+        close(*d->m_fd);
+    }
 }
 
 void PipeWireSourceItem::itemChange(QQuickItem::ItemChange change, const QQuickItem::ItemChangeData &data)
@@ -126,7 +130,10 @@ void PipeWireSourceItem::setFd(uint fd)
     if (fd == d->m_fd)
         return;
 
-    d->m_fd = fcntl(fd, F_DUPFD_CLOEXEC, 3);
+    if (d->m_fd) {
+        close(*d->m_fd);
+    }
+    d->m_fd = fd;
     refresh();
     Q_EMIT fdChanged(fd);
 }
@@ -146,7 +153,7 @@ void PipeWireSourceItem::refresh()
         };
     } else {
         d->m_stream.reset(new PipeWireSourceStream(this));
-        d->m_stream->createStream(d->m_nodeId, d->m_fd);
+        d->m_stream->createStream(d->m_nodeId, d->m_fd.value_or(0));
         if (!d->m_stream->error().isEmpty()) {
             d->m_stream.reset(nullptr);
             d->m_nodeId = 0;
@@ -334,8 +341,7 @@ void PipeWireSourceItem::updateTextureDmaBuf(const DmaBufAttributes &attribs, sp
             eglDestroyImageKHR(display, d->m_image);
         }
         const auto size = d->m_stream->size();
-        const EGLContext context = static_cast<EGLContext>(QGuiApplication::platformNativeInterface()->nativeResourceForIntegration("eglcontext"));
-        d->m_image = GLHelpers::createImage(display, context, attribs, PipeWireSourceStream::spaVideoFormatToDrmFormat(format), size);
+        d->m_image = GLHelpers::createImage(display, attribs, PipeWireSourceStream::spaVideoFormatToDrmFormat(format), size, nullptr);
         if (d->m_image == EGL_NO_IMAGE_KHR) {
             d->m_stream->renegotiateModifierFailed(format, attribs.modifier);
             return nullptr;
@@ -391,7 +397,7 @@ void PipeWireSourceItem::componentComplete()
 
 uint PipeWireSourceItem::fd() const
 {
-    return d->m_fd;
+    return d->m_fd.value_or(0);
 }
 
 uint PipeWireSourceItem::nodeId() const
